function [x,info] = FISTA(A,b,lambda,options)
% FISTA   Fast Iterative Shrinkage-Thresholding Algorithm
%
%    [X,INFO] = FISTA(A,B,LAMBA,OPTIONS) solves
%
%       minimize 1/2 * ||Ax-b||_2^2 + lamba * ||x||_1
%
%    using the fast iterative shrinkage-thresholding algorithm
%    proposed by Beck and Teboulle. A bound on the Lipschitz
%    constant of the gradient is determined using normest of A
%    so no restriction on the norm of A needs to be made. The
%    following options are provided:
%
%    OPTIONS
%       .iterations     maximum number of iterations
%       .verbosity      verbosity level: 0 = quiet, 1 = summary, 2 = all
%       .priduaTol      optimality tolerance for rel. primal-dual gap
%       .lambdaTol      optimality tolerance for rel. lambda gap
%       .maxMatvec      maximum matrix-vector multiplies allowed
%       .maxRuntime     maximum runtime allowed (in seconds)
%       .prefix         prefix string
%       .L              estimation of largest eigenvalue of A'*A
%       .x0             initial solution estimate
%

% Written by: Ewout van den Berg;
% September 28, 2009

% Start timer
t0 = tic;

% Parse options
if nargin < 4 || isempty(options)
    options = struct();
end

% Set default parameters
iterations = 10000;
verbosity  = 1;
priduaTol  = 1e-6;
lambdaTol  = 1e-6;
maxMatvec  = Inf;
maxRuntime = Inf;
prefix     = '';
x0         = A'*b;

if isfield(options,'iterations'), iterations = options.iterations; end
if isfield(options,'verbosity' ), verbosity  = options.verbosity;  end
if isfield(options,'priduaTol' ), priduaTol  = options.priduaTol;  end
if isfield(options,'lambdaTol' ), lambdaTol  = options.lambdaTol;  end
if isfield(options,'maxMatvec' ), maxMatvec  = options.maxMatvec;  end
if isfield(options,'maxRuntime'), maxRuntime = options.maxRuntime; end
if isfield(options,'prefix'    ), prefix     = options.prefix;     end
if isfield(options,'x0'        ), x0         = options.x0;         end

% Estimate Lipschitz constant
if ~isfield(options,'L');
   L = 1.2 * normest(A*A');
else
   L = options.L;
end


% Convert lambda value based on Lipschitz bound
lambdaBar = lambda / L;

% Exit conditions (constants)
EXIT_ITERATIONS    = 1;
EXIT_OPTIMAL       = 2;
EXIT_MATVEC_LIMIT  = 3;
EXIT_RUNTIME_LIMIT = 4;


% ---------------------------------------------------------------------
% Log header
% ---------------------------------------------------------------------
if verbosity > 0
   fprintf('%s%s\n', prefix, repmat('-',40,1));
   fprintf('%s%5s  %9s  %9s  %9s\n',prefix,'iter','objective','priduaGap','lambdaGap');
   fprintf('%s%s\n', prefix, repmat('-',40,1));
end
dispRate = 1; % Frequency of printing


% Initialize
x = x0;  y = x0;  t = 0;
iter = 1; status = 0;

% =====================================================================
% MAIN LOOP
% =====================================================================
while 1,
   yNorm  = norm(y,1);
   r      = A*y - b;
   rtr    = r'*r;
   f      = rtr / 2 + lambdaBar * yNorm;
   Atr    = A' * r;
   AtrInf = norm(Atr,inf);
   
   % Compute relative primal-dual and lambda gaps
   lambdaGap = abs(lambda - AtrInf) / max(1,lambda);
   priduaGap = abs(yNorm + (r'*b + rtr) / AtrInf) / max(1,yNorm);

   % ------------------------------------------------------------------
   % Test exit conditions
   % ------------------------------------------------------------------
   if 2*iter >= maxMatvec
       status = EXIT_MATVEC_LIMIT;
   end
   if toc(t0) >= maxRuntime
       status = EXIT_RUNTIME_LIMIT;
   end
   if iter >= iterations
       status = EXIT_ITERATIONS;
   end
   if (lambdaGap <= lambdaTol) && (priduaGap <= priduaTol)
       status = EXIT_OPTIMAL;
   end

   % ------------------------------------------------------------------
   % Print log and act on exit conditions
   % ------------------------------------------------------------------
   if verbosity > 0
      if ((verbosity == 2) || ...
          (verbosity == 1 && (mod(iter,dispRate) == 0)) || ...
          (status ~= 0))
         fprintf('%s%5i  %9.3e  %9.3e  %9.3e\n', prefix, iter, f, priduaGap,lambdaGap);
         if round(iter / dispRate) == 20
             dispRate = dispRate * 10;
         end         
      end
   end

   if status ~= 0
      break;
   end
   
   % ------------------------------------------------------------------
   % Iterations begin here
   % ------------------------------------------------------------------
   iter  = iter + 1;
   xprev = x;
   v     = y - Atr / L;
   x     = sign(v) .* max(abs(v) - lambdaBar, 0);
   tprev = t;
   t     = (1 + sqrt(1+4*t^2)) / 2;
   y     = x + ((tprev - 1) / t) * (x - xprev);
end

% Determine status message
switch status
   case EXIT_ITERATIONS
      statusMsg = 'ERROR EXIT -- Too many iterations';
   case EXIT_OPTIMAL
      statusMsg = 'EXIT -- Optimal solution found';
   case EXIT_MATVEC_LIMIT
      statusMsg = 'EXIT -- Maximum matrix-vector operations reached';
   case EXIT_RUNTIME_LIMIT
      statusMsg = 'EXIT -- Maximum runtime reached';
   otherwise
      statusMsg = 'ERROR -- Unknown terminal condition';
end

if verbosity > 0
   fprintf('%s\n', prefix);
   fprintf('%s%s\n', prefix, statusMsg);
end

% Prepare output variables
info = struct();
info.r         = r;
info.xNorm     = yNorm; % [sic]
info.rNorm     = sqrt(rtr);
info.priduaGap = priduaGap;
info.lambdaGap = lambdaGap;
info.iter      = iter;
info.timeTotal = toc(t0);
info.status    = status;
info.statusMsg = statusMsg;
info.L         = L;

x = y;
