function [X,info] = FISTA_MMV(A,B,lambda,options)
% FISTA   Fast Iterative Shrinkage-Thresholding Algorithm
%
%    [X,INFO] = FISTA(A,B,LAMBA,OPTIONS) solves
%
%       minimize 1/2 * ||AX-B||_F^2 + lamba * ||X||_1,2
%
%    using the fast iterative shrinkage-thresholding algorithm
%    proposed by Beck and Teboulle. A bound on the Lipschitz
%    constant of the gradient is determined using normest of A
%    so no restriction on the norm of A needs to be made. The
%    following options are provided:
%
%    OPTIONS
%       .iterations     maximum number of iterations
%       .verbosity      verbosity level: 0 = quiet, 1 = summary, 2 = all
%       .priduaTol      optimality tolerance for rel. primal-dual gap
%       .lambdaTol      optimality tolerance for rel. lambda gap
%       .prefix         prefix string
%       .L              estimation of largest eigenvalue of A'*A
%       .X0             initial solution estimate
%

% Written by: Ewout van den Berg;
% October 1, 2009

% Start timer
t0 = tic;

% Parse options
if nargin < 4 || isempty(options)
    options = struct();
end

% Set default parameters
iterations = 10000;
verbosity  = 1;
priduaTol  = 1e-6;
lambdaTol  = 1e-6;
prefix     = '';
X0         = A'*B;

if isfield(options,'iterations')
   iterations = options.iterations;
end
if isfield(options,'verbosity')
    verbosity = options.verbosity;
end
if isfield(options,'priduaTol')
    priduaTol = options.priduaTol;
end
if isfield(options,'lambdaTol')
    lambdaTol = options.lambdaTol;
end
if isfield(options,'prefix')
    prefix = options.prefix;
end
if isfield(options,'X0')
    X0 = options.X0;
end

% Estimate Lipschitz constant
if ~isfield(options,'L');
   L = 1.2 * normest(A*A');
else
   L = options.L;
end

% Convert lambda value based on Lipschitz bound
lambdaBar = lambda / L;

% Exit conditions (constants)
EXIT_ITERATIONS = 1;
EXIT_OPTIMAL    = 2;


% ---------------------------------------------------------------------
% Log header
% ---------------------------------------------------------------------
if verbosity > 0
   fprintf('%s%s\n', prefix, repmat('-',40,1));
   fprintf('%s%5s  %9s  %9s  %9s\n',prefix,'iter','objective','priduaGap','lambdaGap');
   fprintf('%s%s\n', prefix, repmat('-',40,1));
end
dispRate = 1; % Frequency of printing


% Initialize
n = size(X0,1);
D = spdiags(zeros(n,1),0,n,n); Didx = 1 + (0:n-1) * (n+1);
X = X0;  Y = X0;  t = 0;
iter = 1; status = 0;

% =====================================================================
% MAIN LOOP
% =====================================================================
while 1,
   YNorm  = sum(sqrt(sum(abs(Y).^2,2)));
   R      = A*Y - B;
   RtR    = sum(sum(abs(R).^2));
   f      = RtR / 2 + lambdaBar * YNorm;
   AtR    = A' * R;
   AtRInf = max(sqrt(sum(abs(AtR).^2,2)));
   
   % Compute relative primal-dual and lambda gaps
   lambdaGap = abs(lambda - AtRInf) / max(1,lambda);
   priduaGap = abs(YNorm + (sum(sum(conj(R).*B)) + RtR) / AtRInf) / max(1,YNorm);

   % ------------------------------------------------------------------
   % Test exit conditions
   % ------------------------------------------------------------------
   if iter >= iterations
       status = EXIT_ITERATIONS;
   end
   if (lambdaGap <= lambdaTol) && (priduaGap <= priduaTol)
       status = EXIT_OPTIMAL;
   end

   % ------------------------------------------------------------------
   % Print log and act on exit conditions
   % ------------------------------------------------------------------
   if verbosity > 0
      if ((verbosity == 2) || ...
          (verbosity == 1 && (mod(iter,dispRate) == 0)) || ...
          (status ~= 0))
         fprintf('%s%5i  %9.3e  %9.3e  %9.3e\n', prefix, iter, f, priduaGap,lambdaGap);
         if round(iter / dispRate) == 20
             dispRate = dispRate * 10;
         end         
      end
   end

   if status ~= 0
      break;
   end
   
   % ------------------------------------------------------------------
   % Iterations begin here
   % ------------------------------------------------------------------
   iter    = iter + 1;
   Xprev   = X;
   V       = Y - AtR / L;
   Vn      = sqrt(sum(abs(V).^2,2));
   Vn(Vn < lambdaBar) = lambdaBar/2;
   D(Didx) = max(1 - lambdaBar ./ Vn, 0);
   X        = D * V;
   tprev    = t;
   t        = (1 + sqrt(1+4*t^2)) / 2;
   Y        = X + ((tprev - 1) / t) * (X - Xprev);
end

% Determine status message
switch status
   case EXIT_ITERATIONS
      statusMsg = 'ERROR EXIT -- Too many iterations';
   case EXIT_OPTIMAL
      statusMsg = 'EXIT -- Optimal solution found';
    otherwise
      statusMsg = 'ERROR -- Unknown terminal condition';
end

if verbosity > 0
   fprintf('%s\n', prefix);
   fprintf('%s%s\n', prefix, statusMsg);
end

% Prepare output variables
info = struct();
info.R         = R;
info.XNorm     = YNorm; % [sic]
info.RNorm     = sqrt(RtR);
info.priduaGap = priduaGap;
info.lambdaGap = lambdaGap;
info.iter      = iter;
info.timeTotal = toc(t0);
info.status    = status;
info.statusMsg = statusMsg;
info.L         = L;

X = Y;
