function [X,info] = FISTA_NUCNORM(M,idx,lambda,options)
% FISTA_NUCNORM   Fast Iterative Shrinkage-Thresholding Algorithm
%
%    [X,INFO] = FISTA_NUCNORM(M,IDX,LAMBA,OPTIONS) solves
%
%       minimize 1/2 * ||X(idx)-M(idx)||_2^2 + lamba * ||X||
%
%    using the fast iterative shrinkage-thresholding algorithm
%    proposed by Beck and Teboulle. The following options are
%    provided:
%
%    OPTIONS
%       .iterations     maximum number of iterations
%       .verbosity      verbosity level: 0 = quiet, 1 = summary, 2 = all
%       .priduaTol      optimality tolerance for rel. primal-dual gap
%       .lambdaTol      optimality tolerance for rel. lambda gap
%       .maxMatvec      maximum matrix-vector multiplies allowed
%       .maxRuntime     maximum runtime allowed (in seconds)
%       .prefix         prefix string
%       .X0             initial solution estimate
%

% Written by: Ewout van den Berg;
% September 28, 2009

% Start timer
t0 = tic;

% Parse options
if nargin < 4 || isempty(options)
    options = struct();
end

% Set default parameters
iterations = 10000;
verbosity  = 1;
priduaTol  = 1e-6;
lambdaTol  = 1e-6;
maxMatvec  = Inf;
maxRuntime = Inf;
prefix     = '';
X0         = zeros(size(M));

if isfield(options,'iterations'), iterations = options.iterations; end
if isfield(options,'verbosity' ), verbosity  = options.verbosity;  end
if isfield(options,'priduaTol' ), priduaTol  = options.priduaTol;  end
if isfield(options,'lambdaTol' ), lambdaTol  = options.lambdaTol;  end
if isfield(options,'maxMatvec' ), maxMatvec  = options.maxMatvec;  end
if isfield(options,'maxRuntime'), maxRuntime = options.maxRuntime; end
if isfield(options,'prefix'    ), prefix     = options.prefix;     end
if isfield(options,'X0'        ), X0         = options.X0;         end

% Estimate Lipschitz constant
L = 1;

% Convert lambda value based on Lipschitz bound
lambdaBar = lambda / L;

% Exit conditions (constants)
EXIT_ITERATIONS    = 1;
EXIT_OPTIMAL       = 2;
EXIT_MATVEC_LIMIT  = 3;
EXIT_RUNTIME_LIMIT = 4;


% ---------------------------------------------------------------------
% Log header
% ---------------------------------------------------------------------
if verbosity > 0
   fprintf('%s%s\n', prefix, repmat('-',40,1));
   fprintf('%s%5s  %9s  %9s  %9s\n',prefix,'iter','objective','priduaGap','lambdaGap');
   fprintf('%s%s\n', prefix, repmat('-',40,1));
end
dispRate = 1; % Frequency of printing


% Initialize
B = zeros(size(M)); B(idx) = M(idx);
R = zeros(size(M));
X = X0;  Y = X0;  t = 0;
iter = 1; status = 0;

% =====================================================================
% MAIN LOOP
% =====================================================================
while 1,
   [U,S,V] = svd(Y);
   s       = diag(S);
   YNorm   = norm(s,1);
   R(idx)  = Y(idx) - M(idx);
   RtR     = sum(sum(abs(R).^2));
   f       = RtR / 2 + lambdaBar * YNorm;
   AtR     = R; % R already has zero entries outside idx
   AtRInf  = norm(R);
  
   % Compute relative primal-dual and lambda gaps
   lambdaGap = abs(lambda - AtRInf) / max(1,lambda);
   priduaGap = abs(YNorm + (sum(sum(conj(R).*B)) + RtR) / AtRInf) / max(1,YNorm);

   % ------------------------------------------------------------------
   % Test exit conditions
   % ------------------------------------------------------------------
   if 2*iter >= maxMatvec
       status = EXIT_MATVEC_LIMIT;
   end
   if toc(t0) >= maxRuntime
       status = EXIT_RUNTIME_LIMIT;
   end
   if iter >= iterations
       status = EXIT_ITERATIONS;
   end
   if (lambdaGap <= lambdaTol) && (priduaGap <= priduaTol)
       status = EXIT_OPTIMAL;
   end

   % ------------------------------------------------------------------
   % Print log and act on exit conditions
   % ------------------------------------------------------------------
   if verbosity > 0
      if ((verbosity == 2) || ...
          (verbosity == 1 && (mod(iter,dispRate) == 0)) || ...
          (status ~= 0))
         fprintf('%s%5i  %9.3e  %9.3e  %9.3e\n', prefix, iter, f, priduaGap,lambdaGap);
         if round(iter / dispRate) == 20
             dispRate = dispRate * 10;
         end         
      end
   end

   if status ~= 0
      break;
   end
   
   % ------------------------------------------------------------------
   % Iterations begin here
   % ------------------------------------------------------------------
   iter   = iter + 1;
   Xprev  = X;
   V      = Y - AtR / L;
   [U,S,V]= svd(V);
   v      = max(diag(S) - lambdaBar, 0);
   X      = U * diag(v) * V';
   tprev  = t;
   t      = (1 + sqrt(1+4*t^2)) / 2;
   Y      = X + ((tprev - 1) / t) * (X - Xprev);
end

% Determine status message
switch status
   case EXIT_ITERATIONS
      statusMsg = 'ERROR EXIT -- Too many iterations';
   case EXIT_OPTIMAL
      statusMsg = 'EXIT -- Optimal solution found';
   case EXIT_MATVEC_LIMIT
      statusMsg = 'EXIT -- Maximum matrix-vector operations reached';
   case EXIT_RUNTIME_LIMIT
      statusMsg = 'EXIT -- Maximum runtime reached';
   otherwise
      statusMsg = 'ERROR -- Unknown terminal condition';
end

if verbosity > 0
   fprintf('%s\n', prefix);
   fprintf('%s%s\n', prefix, statusMsg);
end

% Prepare output variables
info = struct();
info.R         = R;
info.XNorm     = YNorm; % [sic]
info.RNorm     = sqrt(RtR);
info.priduaGap = priduaGap;
info.lambdaGap = lambdaGap;
info.iter      = iter;
info.timeTotal = toc(t0);
info.status    = status;
info.statusMsg = statusMsg;
info.L         = L;

X = Y;
