function [x,info] = bpdn_cvx(A,b,sigma,lambdas,options)
% Solve using CVX/SDPT3:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% References:
% * Michael Grant and Stephen Boyd
%   CVX: Matlab software for disciplined convex programming.
%   http://www.stanford.edu/~boyd/cvx/
% * Kim-Chuan Toh, Michael J. Todd, and Reha H. Tutuncu
%   SDPT3 - A Matlab software package for semidefinite programming.
%   http://www.math.nus.edu.sg/~mattohkc/sdpt3.html

% August 17, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Parse parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

if isfield(options,'verbosity') && options.verbosity > 0
   options.verbosity = true;
else
   options.verbosity = false;
end

% ---------------------------------------------------------------------
% Configure CVX
% ---------------------------------------------------------------------
solver = cvx_solver;     cvx_solver('SDPT3');
prec   = cvx_precision;  cvx_precision('default');
verb   = cvx_quiet(~options.verbosity);
cvx_clear; % Just in case

t0 = tic;
if isreal(A) && (isreal(b))
% =====================================================================
cvx_begin
   variable x(size(A,2));
   minimize norm(x,1);
   subject to
      norm(A*x-b,2) <= sigma;
cvx_end
% =====================================================================
else
% =====================================================================
cvx_begin
   variable x(size(A,2)) complex;
   minimize norm(x,1);
   subject to
      norm(A*x-b,2) <= sigma;
cvx_end
% =====================================================================
end
info.time = toc(t0);


% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = NaN;
info.nAT    = NaN;
info.nIter  = NaN;
info.r      = b - A*x;
info.lambda = norm(A'*info.r,inf);
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);

% ---------------------------------------------------------------------
% Restore original CVX configuration
% ---------------------------------------------------------------------
cvx_solver(solver);
cvx_precision(prec);
cvx_quiet(verb);

