function [x,info] = bpdn_fista(A,b,sigma,lambdas,options)
% Solve using FISTA:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% Because FISTA solves only the Lagrangian formulation
%
%    Minimize (lambda/2) * ||Ax-b||_2^2 + ||x||_1
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Amir Beck and Marc Teboulle
%   A fast iterative shrinkage-thresholding algorithm with application
%   to wavelet-based image deblurring

% September 28, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

if ~isfield(options,'verbosity' ), options.verbosity  = 0;    end;
if ~isfield(options,'iterations'), options.iterations = 1e4;  end;
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf;  end;
if ~isfield(options,'priduaGap' ), options.priduaGap  = 1e-5; end;
if ~isfield(options,'lambdaGap' ), options.lambdaGap  = 1e-5; end;


% ---------------------------------------------------------------------
% Solve problem
% ---------------------------------------------------------------------
op = spottify(A);
nIter = 0;


t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   [x,data] = FISTA(op,b,lambdas(i),options);
   nIter = nIter + data.iter;

   % Next initial point
   options.x0 = x;
   options.L  = data.L; % Reuse Lipschitz estimate
end
% =====================================================================
info.time = toc(t0);


% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = op.counter.mode1;
info.nAT    = op.counter.mode2;
info.nIter  = nIter;
info.r      = b - A*x;
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);
info.lambda = lambdas(end);
info.L      = data.L;

