function [x,info] = bpdn_fpc(A,b,sigma,lambdas,options)
% Solve using FPC version 2.0:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% Because FPC solves only the Lagrangian formulation
%
%    Minimize (lambda/2) * ||Ax-b||_2^2 + ||x||_1
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Elaine T. Hale, Wotao Yin, and Yin Zhang
%   A fixed-point continuation method for l1-regularized minimization with
%   applications to compressed sensing.
%   http://www.caam.rice.edu/~optimization/L1/fpc/

% August 18, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf;  end;

opts = struct();
opts.maxRuntime = options.maxRuntime;

if ~isreal(A) || ~isreal(b)
   error('bpdn_fpc does not support complex data');
end

% ---------------------------------------------------------------------
% Scale problem once before calling FPC
% ---------------------------------------------------------------------
opts.scale = false;
n          = size(A,2);
op         = spottify(A);
Afun       = @(x,mode) explicitMatrix(op,x,mode);
[scale,Abar,bbar] = scaleProblem(n,1,Afun,b,[]);


% ---------------------------------------------------------------------
% Solve problem
% ---------------------------------------------------------------------
nIter = 0;


t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   data  = fpc_bb(n, Abar, bbar, scale / lambdas(i), [], opts);
   x     = data.x;
   nIter = nIter + data.itr;

   % Next initial point
   opts.x0 = x;
end
% =====================================================================
info.time = toc(t0);


% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = op.counter.mode1;
info.nAT    = op.counter.mode2;
info.nIter  = nIter;
info.r      = b - A*x;
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);
info.lambda = lambdas(end);

