function [x,info] = bpdn_fpcas(A,b,sigma,lambdas,options)
% Solve using FPC_AS version 1.1:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% Because FPC_AS solves only the Lagrangian formulation
%
%    Minimize 0.5 ||Ax-b||_2^2 + lambda ||x||_1
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Zaiwen Wen, Wotao Yin, Donald Goldfarb, and Yin Zhang
%   A fast algorithm for sparse reconstruction based on shrinkage,
%   subspace optimization and continuation.
%   http://www.caam.rice.edu/~optimization/L1/FPC_AS/

% August 18, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;
opts = struct();
if isfield(options,'verbosity') && options.verbosity > 0
   opts.PrintOptions = 1;
else
   opts.PrintOptions = 0;
end

% ---------------------------------------------------------------------
% Set default parameters explicitly
% ---------------------------------------------------------------------
n  = size(A,2);
op = spottify(A);

t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   x = FPC_AS(n,op,b,lambdas(i),[],opts);
   opts.x0 = x;
end
% =====================================================================
info.time = toc(t0);

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = op.counter.mode1;
info.nAT    = op.counter.mode2;
info.nIter  = NaN;
info.r      = b - A*x;
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);
info.lambda = lambdas(end);

