function [x,info] = bpdn_gpsr(A,b,sigma,lambdas,options)
% Solve using GPSR v6.0:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% Because GPSR solves only the Lagrangian formulation
%
%    Minimize 0.5 ||Ax-b||_2^2 + lambda ||x||_1
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Mario Figueiredo, Robert D. Nowak, and Stephen  J. Wright
%   Gradient projection for sparse reconstruction: application to
%   compressed sensing and other inverse problems
%   http://www.lx.it.pt/~mtf/GPSR/

% August 18, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;
opts = {};
if isfield(options,'verbosity') && options.verbosity > 0
   verbosity  = 1;
else
   verbosity  = 0;
end
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf; end;

opts{end+1} = 'Verbose';
opts{end+1} = verbosity;
opts{end+1} = 'ToleranceA';
opts{end+1} = 1e-5;
opts{end+1} = 'MaxRuntime';
opts{end+1} = options.maxRuntime;

if ~isreal(A) || ~isreal(b)
   error('bpdn_gpsr does not support complex data');
end

% ---------------------------------------------------------------------
% Set default parameters explicitly
% ---------------------------------------------------------------------
optsBase = opts;
ATy      = norm(A'*b,inf);
op       = spottify(A);

t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   opts = optsBase;
   if lambdas(i) <= 0.05 * ATy
      opts{end+1} = 'Continuation';
      opts{end+1} = 1;
      if i > 1
         opts{end+1} = 'FirstTauFactor';
         opts{end+1} = lambdas(i-1);
      end
   end

   if i==1
      x = GPSR_BB(b,op,lambdas(i),opts{:});
   else
      x = GPSR_BB(b,op,lambdas(i),'Initialization',x,opts{:});
   end
end
% =====================================================================
info.time = toc(t0);

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = op.counter.mode1;
info.nAT    = op.counter.mode2;
info.nIter  = NaN;
info.r      = b - A*x;
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);
info.lambda = lambdas(end);

