function [x,info] = bpdn_l1ls(A,b,sigma,lambdas,options)
% Solve using L1LS:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% Because L1LS solves only the Lagrangian formulation
%
%    Minimize ||Ax-b||_2^2 + lambda ||x||_1
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Seung-Jean Kim, Kwang-Moo Koh, Stephen Boyd, and Dimitry Gorinevsky
%   A method for large-scale l1-regularized least squares
%   http://www.stanford.edu/~boyd/l1_ls/

% August 18, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;
if isfield(options,'verbosity') && options.verbosity > 0
   quiet = 0;
else
   quiet = 1;
end

if ~isreal(A) || ~isreal(b)
   error('bpdn_l1ls does not support complex data');
end

% ---------------------------------------------------------------------
% Solve problem
% ---------------------------------------------------------------------

t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   x = l1_ls(A,b,lambdas(i)*2,1e-3,quiet);
end
% =====================================================================
info.time = toc(t0);

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = NaN;
info.nAT    = NaN;
info.nIter  = NaN;
info.r      = b - A*x;
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);
info.lambda = lambdas(end);

