function [x,info] = bpdn_nesta(A,b,sigma,lambdas,options)
% Solve using NESTA:
%
%    Minimize   ||x||_1   subject to    ||Ax-b||_2 <= sigma
%
% where A has orthonormal rows.
%
% References:
% * Stephen Becker, Jerome Bobin, and Emmanuel J. Candes
%   NESTA: A fast and accurate first-order method for sparse recovery,
%   http://www.acm.caltech.edu/~nesta/

% August 17, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf; end;


opts = struct();
if isfield(options,'x0' ), opts.xplug = options.x0;  end;
if isfield(options,'fid'), opts.fid   = options.fid; end;
if isfield(options,'verbosity') && options.verbosity > 0
   opts.Verbose = 1;
else
   opts.Verbose = 0;
end

% ---------------------------------------------------------------------
% Set default parameters explicitly
% ---------------------------------------------------------------------
opts.maxRuntime = options.maxRuntime;
opts.MaxIntIter = 5;      % number of continuation steps
opts.maxiter    = 10000;  % max number of iterations in an inner loop
opts.TolVar     = 1e-5;   % tolerance for the stopping criteria
opts.TypeMin    = 'L1';   % minimize smoothed version of the l_1 norm
opts.stopTest   = 1;      % stop when the relative change in the
                          % objective function is less than TolVar

mu = 1e-6; % Default 0.02, see paper.
op = spottify(A);

t0 = tic;
% =====================================================================
[x,nIter,r,output] = NESTA(op, op', b, mu, sigma, opts);
% =====================================================================
info.time = toc(t0);

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = op.counter.mode1;
info.nAT    = op.counter.mode2;
info.nIter  = nIter;
info.r      = b - A*x;
info.rNorm  = norm(info.r,2);
info.xNorm  = norm(x,1);
info.lambda = norm(A'*info.r,inf);

