function [X,info] = mmv_cvx(A,B,sigma,lambdas,options)
% Solve using CVX/SDPT3:
%
%    Minimize   ||X||_1,2   subject to    ||AX-B||_F <= sigma
%
% References:
% * Michael Grant and Stephen Boyd
%   CVX: Matlab software for disciplined convex programming.
%   http://www.stanford.edu/~boyd/cvx/
% * Kim-Chuan Toh, Michael J. Todd, and Reha H. Tutuncu
%   SDPT3 - A Matlab software package for semidefinite programming.
%   http://www.math.nus.edu.sg/~mattohkc/sdpt3.html

% August 21, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Parse parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

if isfield(options,'verbosity') && options.verbosity > 0
   options.verbosity = true;
else
   options.verbosity = false;
end

% ---------------------------------------------------------------------
% Configure CVX
% ---------------------------------------------------------------------
solver = cvx_solver;     cvx_solver('SDPT3');
prec   = cvx_precision;  cvx_precision('default');
verb   = cvx_quiet(~options.verbosity);
cvx_clear; % Just in case


t0 = tic;
if isreal(A) && (isreal(B))
% =====================================================================
if sigma == 0
   cvx_begin
      variable X(size(A,2),size(B,2));
      minimize(sum(norms(X,2,2)));
      subject to
         A*X == B;
   cvx_end
else
   cvx_begin
      variable X(size(A,2),size(B,2));
      minimize(sum(norms(X,2,2)));
      subject to
         norm(A*X-B,'fro') <= sigma;
   cvx_end
end
% =====================================================================
else
% =====================================================================
if sigma == 0
   cvx_begin
      variable X(size(A,2),size(B,2)) complex;
      minimize(sum(norms(X,2,2)));
      subject to
         A*X == B;
   cvx_end
else
   cvx_begin
      variable X(size(A,2),size(B,2)) complex;
      minimize(sum(norms(X,2,2)));
      subject to
         norm(A*X-B,'fro') <= sigma;
   cvx_end
end
% =====================================================================
end

info.time = toc(t0);


% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA     = NaN;
info.nAT    = NaN;
info.nIter  = NaN;
info.R      = B - A*X;
info.lambda = max(norms(A'*info.R,2,2));
info.RNorm  = norm(info.R,'fro');
info.XNorm  = sum(norms(X,2,2));

% ---------------------------------------------------------------------
% Restore original CVX configuration
% ---------------------------------------------------------------------
cvx_solver(solver);
cvx_precision(prec);
cvx_quiet(verb);

