function [X,info] = mmv_fista(A,B,sigma,lambdas,options)
% Solve using FISTA_MMV:
%
%    Minimize   ||X||_1,2   subject to    ||AX-B||_F <= sigma
%
% Because FISTA_MMV solves only the Lagrangian formulation
%
%    Minimize (lambda/2) * ||AX-B||_F^2 + ||X||_1,2
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Amir Beck and Marc Teboulle
%   A fast iterative shrinkage-thresholding algorithm with application
%   to wavelet-based image deblurring

% October 5, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

if ~isfield(options,'verbosity' ), options.verbosity  = 0;    end;
if ~isfield(options,'iterations'), options.iterations = 1e4;  end;
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf;  end;
if ~isfield(options,'priduaGap' ), options.priduaGap  = 1e-5; end;
if ~isfield(options,'lambdaGap' ), options.lambdaGap  = 1e-5; end;


% ---------------------------------------------------------------------
% Solve problem
% ---------------------------------------------------------------------
op = spottify(A);
nIter = 0;


t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   [X,data] = FISTA_MMV(op,B,lambdas(i),options);
   nIter = nIter + data.iter;

   % Next initial point
   options.X0 = X;
   options.L  = data.L; % Reuse Lipschitz estimate
end
% =====================================================================
info.time = toc(t0);


% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nA        = op.counter.mode1;
info.nAT       = op.counter.mode2;
info.nIter     = nIter;
info.R         = B - A*X;
info.RNorm     = norm(info.R,'fro');
info.XNorm     = sum(sqrt(sum(abs(X).^2,2)));
info.lambda    = lambdas(end);
info.priduaGap = data.priduaGap;
info.lambdaGap = data.lambdaGap;
info.L         = data.L;

