function [X,info] = mmv_sparsa(A,B,sigma,lambdas,options)
% Solve using SpaRSA,
%
%    Minimize   1/2 * ||AX-B||_F^2 + lambda * ||X||_1,2
%
% which is a penalized formulation of
%
%    Minimize   ||X||_1,2   subject to    ||AX-B||_F <= sigma
%
% References:
% * Stephen J. Wright, Robert D. Nowak, and  Mario A.T. Figueiredo
%   Sparse reconstruction by separable approximation

% October 5, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Parse parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

opts = {};
if isfield(options,'verbosity') && options.verbosity > 0
   verbosity  = 1;
else
   verbosity  = 0;
end
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf; end;

opts{end+1} = 'Verbose';
opts{end+1} = verbosity;
opts{end+1} = 'ToleranceA';
opts{end+1} = 1e-5;
opts{end+1} = 'StopCriterion';
opts{end+1} = 2;


% ---------------------------------------------------------------------
% Prepare for solve
% ---------------------------------------------------------------------
optsBase = opts;

[m,n] = size(A);
[m,k] = size(B);

op     = spottify(A);
groups = reshape(repmat([1:n]',1,k),n*k,1);
psi    = @(x,lambda) group_vector_soft(x,lambda,groups);
phi    = @(x) group_l2norm(x,groups);
Afun   = @(x) funMMVSparsaA(op,x,m,n,k);
ATfun  = @(x) funMMVSparsaAT(op,x,m,n,k);

optsBase{end+1} = 'AT';
optsBase{end+1} = ATfun;
optsBase{end+1} = 'Psi';
optsBase{end+1} = psi;
optsBase{end+1} = 'Phi';
optsBase{end+1} = phi;


t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   opts = optsBase;

   if i==1
      x = SpaRSA(B(:),Afun,lambdas(i),opts{:});
   else
      x = SpaRSA(B(:),Afun,lambdas(i),'Initialization',x,opts{:});
   end
end
% =====================================================================
info.time = toc(t0);

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
X = reshape(x,n,k);
info.nA     = op.counter.mode1;
info.nAT    = op.counter.mode2;
info.R      = B - A*X;
info.RNorm  = norm(info.R,'fro');
info.XNorm  = sum(sqrt(sum(abs(X).^2,2)));
info.lambda = lambdas(end);


function y = funMMVSparsaA(op,x,m,n,k)
y = reshape(op*reshape(x,n,k),m*k,1);

function y = funMMVSparsaAT(op,x,m,n,k)
y = reshape(op'*reshape(x,m,k),n*k,1);

