function [X,info] = nucnorm_cvx(M,idx,sigma,lambdas,options)
% Solve using CVX/SDPT3:
%
%    Minimize   ||X||_*   subject to    ||X(idx)-M(idx)||_2 <= sigma
%
% References:
% * Michael Grant and Stephen Boyd
%   CVX: Matlab software for disciplined convex programming.
%   http://www.stanford.edu/~boyd/cvx/
% * Kim-Chuan Toh, Michael J. Todd, and Reha H. Tutuncu
%   SDPT3 - A Matlab software package for semidefinite programming.
%   http://www.math.nus.edu.sg/~mattohkc/sdpt3.html

% August 19, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Parse parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

if isfield(options,'verbosity')
   if options.verbosity > 0
      options.verbosity = false;
   else
      options.verbosity = true;
   end
else
   options.verbosity = true;
end

% ---------------------------------------------------------------------
% Configure CVX
% ---------------------------------------------------------------------
solver = cvx_solver;     cvx_solver('SDPT3');
prec   = cvx_precision;  cvx_precision('default');
verb   = cvx_quiet(options.verbosity);
cvx_clear; % Just in case


try, t0 = toc; catch err, t0 = 0; tic; end;
% =====================================================================
if sigma == 0
   cvx_begin
      variable X(size(M));
      minimize norm_nuc(X);
      subject to
         X(idx) == M(idx);
   cvx_end
else
   cvx_begin
      variable X(size(M));
      minimize norm_nuc(X);
      subject to
         norm(X(idx)-M(idx),2) <= sigma;
   cvx_end
end
% =====================================================================
info.time   = toc - t0;

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nIter  = NaN;
info.r      = X(idx) - M(idx);
info.xNorm  = norm_nuc(full(X));
info.rNorm  = norm(info.r,2);

% ---------------------------------------------------------------------
% Restore original CVX configuration
% ---------------------------------------------------------------------
cvx_solver(solver);
cvx_precision(prec);
cvx_quiet(verb);

