function [X,info] = nucnorm_fista(M,idx,sigma,lambdas,options)
% Solve using FISTA_NUCNORM:
%
%    Minimize   ||X||   subject to    ||M(idx)-X(idx)||_F <= sigma
%
% Because FISTA_NUCNORM solves only the Lagrangian formulation
%
%    Minimize (lambda/2) * ||M(idx)-X(idX)||_F^2 + ||X||
%
% We give a sequence of lambda values that correspond to the root finding
% procedure (if there would be one for the latter formulation).
%
% References:
% * Amir Beck and Marc Teboulle
%   A fast iterative shrinkage-thresholding algorithm with application
%   to wavelet-based image deblurring

% October 5, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Convert parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;

if ~isfield(options,'verbosity' ), options.verbosity  = 0;    end;
if ~isfield(options,'iterations'), options.iterations = 1e4;  end;
if ~isfield(options,'maxRuntime'), options.maxRuntime = Inf;  end;
if ~isfield(options,'priduaGap' ), options.priduaGap  = 1e-5; end;
if ~isfield(options,'lambdaGap' ), options.lambdaGap  = 1e-5; end;


% ---------------------------------------------------------------------
% Solve problem
% ---------------------------------------------------------------------
nIter = 0;


t0 = tic;
% =====================================================================
for i=1:length(lambdas)
   [X,data] = FISTA_NUCNORM(M,idx,lambdas(i),options);
   nIter = nIter + data.iter;

   % Next initial point
   options.X0 = X;
end
% =====================================================================
info.time = toc(t0);


% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
info.nIter     = nIter;
info.nProj     = nIter;
info.r         = zeros(size(M));
info.r(idx)    = M(idx) - X(idx);
info.rNorm     = norm(info.r,'fro');
info.xNorm     = norm_nuc(X);
info.lambda    = lambdas(end);
info.priduaGap = data.priduaGap;
info.lambdaGap = data.lambdaGap;
info.L         = data.L;

