function [X,info] = nucnorm_fpc(M,idx,sigma,lambdas,options)
% Solve using FPC:
%
%    Minimize   ||X||_*   subject to    X(idx) = M(idx)
%
% References:
% * Jian-Feng Cai, Emmanuel J. Candès and Zuowei Shen
%   A singular value thresholding algorithm for matrix completion
%   http://svt.caltech.edu/
% * Shiqian Ma, Donald Goldfarb, and Lifeng Chen
%   Fixed point and Bregman iterative methods for matrix rank
%   minimization.
%   http://www.columbia.edu/~sm2756/FPCA.htm

% August 19, 2009: Ewout van den Berg

% ---------------------------------------------------------------------
% Initialize options structure
% ---------------------------------------------------------------------
opts          = struct();
opts.mu_final = 0.01;
opts.maxiter  = 500;
opts.tol      = 1e-3;

% ---------------------------------------------------------------------
% Parse parameters
% ---------------------------------------------------------------------
if nargin < 5, options = struct(); end;
if isfield(options,'verbosity') && options.verbosity > 0
   verbosity  = 1;
else
   verbosity  = 0;
end

if isfield(options,'mu')
   opts.mu_final = options.mu;
end
if isfield(options,'iterations')
   opts.maxiter = options.iterations;
end
if isfield(options,'tolerance')
   opts.tol = options.tolerance;
end

% ---------------------------------------------------------------------
% Solve problem
% ---------------------------------------------------------------------
[m,n] = size(M);


t0 = tic;
% =============================================================================
[U,S,V,iter] = FPC([m,n], idx, M(idx), opts.mu_final, opts.maxiter, opts.tol, verbosity);
% =============================================================================
info.time = toc(t0);

% ---------------------------------------------------------------------
% Gather statistics
% ---------------------------------------------------------------------
X = U*S*V';
info.nIter = iter;
info.nProj = iter;
info.r     = X(idx) - M(idx);
info.xNorm = norm(diag(S),1);
info.rNorm = norm(info.r,2);

