function info = problem_bp(probIdx)
% PROBLEM_BP  Generates BP test problems
%
%    INFO = PROBLEM_BP(IDX) returns problem with index IDX, when IDX is
%    omitted the INFO contains the number of problems.
%
%    INFO has the following fields
%       .A        Matrix or operator
%       .b        Right-hand side
%       .x0       Generating vector (optional)
%       .ortho    Set if A has orthonormal rows
%       .oper     Set if A is an implicit operator
%       .name     Problem name (use this for saving data, number could change)

% September 29, 2009: Ewout van den Berg

info  = struct();
state = rand('state');
staten= randn('state');

if nargin < 1
   info = 47; % Number of problems
   return;
end


switch probIdx

   case {1,'blurspike'}
      data        = generateProblem(702);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'blurspike';

   case {2,'blocksig'}
      data        = generateProblem(2);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = true;
      info.oper   = true;
      info.name   = 'blocksig';

   case {3,'cosspike'}
      data        = generateProblem(3);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'cosspike';

   case {4,'zsinspike'}
      data        = generateProblem(4);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'zsinspike';

   case {5,'gcosspike'}
      data        = generateProblem(5);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'gcosspike';

   case {6,'jitter'}
      data        = generateProblem(902);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = true;
      info.oper   = true;
      info.name   = 'jitter';

   case {7,'sgnspike'}
      data        = generateProblem(7);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'sgnspike';

   case {8,'zsgnspike'}
      data        = generateProblem(8);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'zsgnspike';

   case {9,'blkheavi'}
      data        = generateProblem(9);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'blkheavi';

   case {10,'angiogram'}
      data        = generateProblem(502);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = true;
      info.oper   = true;
      info.name   = 'angiogram';

   case {11,'gausspike'}
      data        = generateProblem(11);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'gausspike';

   case {12,'dcthdr'}
      data        = generateProblem(12);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = true;
      info.oper   = true;
      info.name   = 'dcthdr';

   case {13,'phantom1'}
      data        = generateProblem(501);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = true;
      info.oper   = true;
      info.name   = 'phantom1';

   % ----------------------------------------------------------
   % Convex combination of Gaussian and all ones
   % ----------------------------------------------------------

   case {14,'gaussone00a'}
      [A,x0]     = problemGaussOnes(0,80);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone00a';

   case {15,'gaussone20a'}
      [A,x0]     = problemGaussOnes(20,80);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone20a';

   case {16,'gaussone40a'}
      [A,x0]     = problemGaussOnes(40,80);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone40a';

   case {17,'gaussone60a'}
      [A,x0]     = problemGaussOnes(60,80);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone60a';

   case {18,'gaussone80a'}
      [A,x0]     = problemGaussOnes(80,80);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone80a';



   case {19,'gaussone00b'}
      [A,x0]     = problemGaussOnes(0,180);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone00b';

   case {20,'gaussone20b'}
      [A,x0]     = problemGaussOnes(20,180);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone20b';

   case {21,'gaussone40b'}
      [A,x0]     = problemGaussOnes(40,180);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone40b';

   case {22,'gaussone60b'}
      [A,x0]     = problemGaussOnes(60,180);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone60b';

   case {23,'gaussone80b'}
      [A,x0]     = problemGaussOnes(80,180);
      info.A     = A;
      info.x0    = x0;
      info.b     = info.A * info.x0;
      info.ortho = false;
      info.oper  = false;
      info.name  = 'gaussone80b';







   case {24,'p3poly'}
      data        = generateProblem(6);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'p3poly';

   case {25,'phantom2'}
      data        = generateProblem(503);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = true;
      info.oper   = true;
      info.name   = 'phantom2';

% DO NOT FORGET TO UPDATE PROBLEM COUNT!!!

   case {26,'blknheavi'}
      data        = generateProblem(10);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'blknheavi';

   case {27,'zheavisgn'}
      data        = generateProblem(1);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.ortho  = false;
      info.oper   = true;
      info.name   = 'zheavisgn';



   % ----------------------------------------------------------
   % Orthonormal rows, implicit, noiseless, m/8-sparse
   % ----------------------------------------------------------
   case {28,'orthimpl1a'}
      randn('state',3); rand('state',3);

      scale = 10;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1a';

   case {29,'orthimpl1b'}
      randn('state',4); rand('state',4);

      scale = 11;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1b';

   case {30,'orthimpl1c'}
      randn('state',5); rand('state',5);

      scale = 12;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1c';

   case {31,'orthimpl1d'}
      randn('state',6); rand('state',6);

      scale = 13;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1d';

   case {32,'orthimpl1e'}
      randn('state',7); rand('state',7);

      scale = 14;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1e';

   case {33,'orthimpl1f'}
      randn('state',8); rand('state',8);

      scale = 15;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1f';

   case {34,'orthimpl1g'}
      randn('state',9); rand('state',9);

      scale = 16;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1g';

   case {35,'orthimpl1h'}
      randn('state',10); rand('state',10);

      scale = 17;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1h';

   case {36,'orthimpl1i'}
      randn('state',10); rand('state',10);

      scale = 18;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1i';

   case {37,'orthimpl1j'}
      randn('state',11); rand('state',11);

      scale = 19;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1j';

   case {38,'orthimpl1k'}
      randn('state',12); rand('state',12);

      scale = 20;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl1k';


   % ----------------------------------------------------------
   % Orthonormal rows, implicit, noiseless, m/4-sparse
   % ----------------------------------------------------------
   case {39,'orthimpl2a'}
      randn('state',3); rand('state',3);

      scale = 10;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2a';

   case {40,'orthimpl2b'}
      randn('state',4); rand('state',4);

      scale = 11;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2b';

   case {41,'orthimpl2c'}
      randn('state',5); rand('state',5);

      scale = 12;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2c';

   case {42,'orthimpl2d'}
      randn('state',6); rand('state',6);

      scale = 13;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2d';

   case {43,'orthimpl2e'}
      randn('state',7); rand('state',7);

      scale = 14;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2e';

   case {44,'orthimpl2f'}
      randn('state',8); rand('state',8);

      scale = 15;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2f';

   case {45,'orthimpl2g'}
      randn('state',9); rand('state',9);

      scale = 16;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2g';

   case {46,'orthimpl2h'}
      randn('state',10); rand('state',10);

      scale = 17;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2h';

   case {47,'orthimpl2i'}
      randn('state',11); rand('state',11);

      scale = 18;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2i';

   case {48,'orthimpl2j'}
      randn('state',12); rand('state',12);

      scale = 19;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-4));
      info.b      = info.A * info.x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0;
      info.name   = 'orthimpl2j';

end

% Restore random number generator state
rand('state',state);
randn('state',staten);





% ---------------------------------------------------------------------
function [A,x0] = problemGaussOnes(pct,sparsity)
% ---------------------------------------------------------------------
% Convex combination of Gaussian and all ones with weight in pct.
randn('state',1);
c  = pct / 100;
A  = (1-c) * randn(600,1800) + c * ones(600,1800);
x0 = zeros(1800,1); x0(1:sparsity) = randn(sparsity,1);

% ---------------------------------------------------------------------
function A = randDCTMat(m,n)
% ---------------------------------------------------------------------
p = randperm(n);
A = opRestriction(n,sort(p(1:m))) * opDCT(n);

% ---------------------------------------------------------------------
function x = randnSparseVec(n,k)
% ---------------------------------------------------------------------
x = zeros(n,1);
p = randperm(n);
x(p(1:k)) = randn(k,1);

