function info = problem_bpdn2(probIdx)
% PROBLEM_BPDN2  Generates BPDN test problems
%
%    INFO = PROBLEM_BPDN2(IDX) returns problem with index IDX, when IDX is
%    omitted the INFO contains the number of problems.
%
%    INFO has the following fields
%       .A        Matrix or operator
%       .b        Right-hand side
%       .x0       Generating vector
%       .n        Additive noise vector
%       .ortho    Set if A has orthonormal rows
%       .oper     Set if A is an implicit operator
%       .sigma    Value for sigma
%       .name     Problem name (use this for saving data, number could change)

% September 29, 2009: Ewout van den Berg

info  = struct();
state = rand('state');
staten= randn('state');

if nargin < 1
   info = 40; % Number of problems
   return;
end

% Set default solution accuracy
accuracy = 1e-10;

switch probIdx
   % ----------------------------------------------------------
   % Orthonormal rows, explicit, noisy, 0.1m-sparse
   % ----------------------------------------------------------
   case {1,'orthn1a'}
      randn('state',1); rand('state',1);

      info.A      = randnOrthMat(100,200);
      info.x0     = randnSparseVec(200,10);
      info.b      = info.A * info.x0;
      info.n      = randn(100,1);
      info.n      = info.n * (0.01 * norm(info.b,2) / norm(info.n,2)); % 1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 1.02*norm(info.n,2);
      info.name   = 'orthn1a';

   case {2,'orthn1b'}
      randn('state',1); rand('state',1);

      info.A      = randnOrthMat(100,200);
      info.x0     = randnSparseVec(200,10);
      info.b      = info.A * info.x0;
      info.n      = randn(100,1);
      info.n      = info.n * (0.02 * norm(info.b,2) / norm(info.n,2)); % 2 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 1.02*norm(info.n,2);
      info.name   = 'orthn1b';

   case {3,'orthn1c'}
      randn('state',1); rand('state',1);

      info.A      = randnOrthMat(100,200);
      info.x0     = randnSparseVec(200,10);
      info.b      = info.A * info.x0;
      info.n      = randn(100,1);
      info.n      = info.n * (0.04 * norm(info.b,2) / norm(info.n,2)); % 4 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 1.02*norm(info.n,2);
      info.name   = 'orthn1c';

   case {4,'orthn2a'}
      randn('state',2); rand('state',2);

      info.A      = randnOrthMat(500,1000);
      info.x0     = randnSparseVec(1000,50);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.01 * norm(info.b,2) / norm(info.n,2)); % 1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'orthn2a';

   case {5,'orthn2b'}
      randn('state',2); rand('state',2);

      info.A      = randnOrthMat(500,1000);
      info.x0     = randnSparseVec(1000,50);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.02 * norm(info.b,2) / norm(info.n,2)); % 2 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'orthn2b';

   case {6,'orthn2c'}
      randn('state',2); rand('state',2);

      info.A      = randnOrthMat(500,1000);
      info.x0     = randnSparseVec(1000,50);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.04 * norm(info.b,2) / norm(info.n,2)); % 4 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'orthn2c';


   case {7,'orthn3a'}
      randn('state',3); rand('state',3);

      info.A      = randnOrthMat(1500,4000);
      info.x0     = randnSparseVec(4000,150);
      info.b      = info.A * info.x0;
      info.n      = randn(1500,1);
      info.n      = info.n * (0.01 * norm(info.b,2) / norm(info.n,2)); % 1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'orthn3a';

   case {8,'orthn3b'}
      randn('state',3); rand('state',3);

      info.A      = randnOrthMat(1500,4000);
      info.x0     = randnSparseVec(4000,150);
      info.b      = info.A * info.x0;
      info.n      = randn(1500,1);
      info.n      = info.n * (0.02 * norm(info.b,2) / norm(info.n,2)); % 2 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'orthn3b';

   case {9,'orthn3c'}
      randn('state',3); rand('state',3);

      info.A      = randnOrthMat(1500,4000);
      info.x0     = randnSparseVec(4000,150);
      info.b      = info.A * info.x0;
      info.n      = randn(1500,1);
      info.n      = info.n * (0.04 * norm(info.b,2) / norm(info.n,2)); % 4 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'orthn3c';

   % ----------------------------------------------------------
   % Heaviside
   % ----------------------------------------------------------

   case {10,'heaviside1'}
      randn('state',1); rand('state',1);

      info.A      = double(opHeaviside(128));
      info.x0     = randnSparseVec(128,12);
      info.b      = info.A * info.x0;
      info.n      = randn(128,1);
      info.n      = info.n * (0.01 * norm(info.b,2) / norm(info.n,2)); % 1 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.98*norm(info.n,2);
      info.name   = 'heaviside1';


   % ----------------------------------------------------------
   % Typical Gaussian matrices
   % ----------------------------------------------------------

   case {11,'gauss1a'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,45);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 1.00 * norm(info.n,2);
      info.name   = 'gauss1a';

   case {12,'gauss1b'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,45);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.50 * norm(info.n,2);
      info.name   = 'gauss1b';

   case {13,'gauss1c'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,45);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.15 * norm(info.n,2);
      info.name   = 'gauss1c';

   case {14,'gauss1d'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,45);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 1.5 * norm(info.n,2);
      info.name   = 'gauss1d';



   case {15,'gauss2a'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,100);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 1.00 * norm(info.n,2);
      info.name   = 'gauss2a';

   case {16,'gauss2b'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,100);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.50 * norm(info.n,2);
      info.name   = 'gauss2b';

   case {17,'gauss2c'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,100);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.15 * norm(info.n,2);
      info.name   = 'gauss2c';

   case {18,'gauss2d'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,100);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 1.5 * norm(info.n,2);
      info.name   = 'gauss2d';


   case {19,'gauss3a'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,200);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 1.00 * norm(info.n,2);
      info.name   = 'gauss3a';

   case {20,'gauss3b'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,200);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.50 * norm(info.n,2);
      info.name   = 'gauss3b';

   case {21,'gauss3c'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,200);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 0.15 * norm(info.n,2);
      info.name   = 'gauss3c';

   case {22,'gauss3d'}
      randn('state',1); rand('state',1);

      info.A      = randn(500,1000);
      info.x0     = randnSparseVec(1000,200);
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.08 * norm(info.b,2) / norm(info.n,2)); % 8 Percent
      info.b      = info.b + info.n;
      info.ortho  = false;
      info.oper   = false;
      info.sigma  = 1.5 * norm(info.n,2);
      info.name   = 'gauss3d';


   % ----------------------------------------------------------
   % Convex combination of Gaussian and all ones
   % ----------------------------------------------------------
   case {23,'gaussone00'}
      [A,x0,n]   = problemGaussOne(0);
      info.A     = A;
      info.x0    = x0;
      info.n     = n;
      info.b     = info.A * info.x0 + info.n;
      info.ortho = false;
      info.oper  = false;
      info.sigma = norm(info.n,2);
      info.name  = 'gaussone00';

   case {24,'gaussone20'}
      [A,x0,n]   = problemGaussOne(20);
      info.A     = A;
      info.x0    = x0;
      info.n     = n;
      info.b     = info.A * info.x0 + info.n;
      info.ortho = false;
      info.oper  = false;
      info.sigma = norm(info.n,2);
      info.name  = 'gaussone20';

   case {25,'gaussone40'}
      [A,x0,n]   = problemGaussOne(40);
      info.A     = A;
      info.x0    = x0;
      info.n     = n;
      info.b     = info.A * info.x0 + info.n;
      info.ortho = false;
      info.oper  = false;
      info.sigma = norm(info.n,2);
      info.name  = 'gaussone40';

   case {26,'gaussone60'}
      [A,x0,n]   = problemGaussOne(60);
      info.A     = A;
      info.x0    = x0;
      info.n     = n;
      info.b     = info.A * info.x0 + info.n;
      info.ortho = false;
      info.oper  = false;
      info.sigma = norm(info.n,2);
      info.name  = 'gaussone60';
      accuracy   = 1e-9;

   case {27,'gaussone80'}
      [A,x0,n]   = problemGaussOne(80);
      info.A     = A;
      info.x0    = x0;
      info.n     = n;
      info.b     = info.A * info.x0 + info.n;
      info.ortho = false;
      info.oper  = false;
      info.sigma = norm(info.n,2);
      info.name  = 'gaussone80';
      accuracy   = 1e-9;


   % ----------------------------------------------------------
   % Orthonormal rows, implicit, noiseless, 0.1m-sparse
   % ----------------------------------------------------------
   case {28,'orthimpl1a'}
      randn('state',3); rand('state',3);

      scale = 10;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1a';

   case {29,'orthimpl1b'}
      randn('state',4); rand('state',4);

      scale = 11;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1b';

   case {30,'orthimpl1c'}
      randn('state',5); rand('state',5);

      scale = 12;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1c';

   case {31,'orthimpl1d'}
      randn('state',6); rand('state',6);

      scale = 13;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1d';

   case {32,'orthimpl1e'}
      randn('state',7); rand('state',7);

      scale = 14;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1e';

   case {33,'orthimpl1f'}
      randn('state',8); rand('state',8);

      scale = 15;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1f';

   case {34,'orthimpl1g'}
      randn('state',9); rand('state',9);

      scale = 16;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1g';

   case {35,'orthimpl1h'}
      randn('state',10); rand('state',10);

      scale = 17;
      info.A      = randDCTMat(2^(scale-2),2^scale);
      info.x0     = randnSparseVec(2^scale,2^(scale-5));
      info.b      = info.A * info.x0;
      info.n      = randn(2^(scale-2),1);
      info.n      = info.n * (0.03 * norm(info.b,2) / norm(info.n,2)); % 3 Percent;
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = norm(info.n,2);
      info.name   = 'orthimpl1h';


   % ----------------------------------------------------------
   % Sparco problems
   % ----------------------------------------------------------

   case {36,'srcsep1_s1'} % DONE COMPUTATION
      data        = generateProblem(401);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-1 * norm(info.b,2);
      info.name   = 'srcsep1_s1';
      accuracy    = 1e-10;


   case {37,'seismic_s1'} % DONE COMPUTATION
      data        = generateProblem(901);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-1 * norm(info.b,2);
      info.name   = 'seismic_s1';
      accuracy    = 1e-10;

   case {38,'phantom2_s1'} % DONE COMPUTATION
      data        = generateProblem(503);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 1e-1 * norm(info.b,2);
      info.name   = 'phantom2_s1';
      accuracy    = 1e-10;

   case {39,'finger_s1'} % DONE COMPUTATION
      data        = generateProblem(703);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-1 * norm(info.b,2);
      info.name   = 'finger_s1';
      accuracy    = 1e-10;

   case {40,'p3poly_s1'} % DONE COMPUTATION
      data        = generateProblem(6);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-1 * norm(info.b,2);
      info.name   = 'p3poly_s1';
      accuracy    = 1e-10;




   case {41,'srcsep1_s2'} % DONE COMPUTATION
      data        = generateProblem(401);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-2 * norm(info.b,2);
      info.name   = 'srcsep1_s2';
      accuracy    = 1e-10;

   case {42,'seismic_s2'} % DONE - NEW - NEW
      data        = generateProblem(901);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-2 * norm(info.b,2);
      info.name   = 'seismic_s2';
      accuracy    = 1.0e-10;

   case {43,'phantom2_s2'} % DONE COMPUTATION
      data        = generateProblem(503);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 1e-2 * norm(info.b,2);
      info.name   = 'phantom2_s2';
      accuracy    = 1e-10;

   case {44,'finger_s2'} % DONE COMPUTATION
      data        = generateProblem(703);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-2 * norm(info.b,2);
      info.name   = 'finger_s2';
      accuracy    = 1e-10;

   case {45,'p3poly_s2'} % DONE COMPUTATION
      data        = generateProblem(6);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-2 * norm(info.b,2);
      info.name   = 'p3poly_s2';
      accuracy    = 1e-10;


   case {46,'srcsep1_s3'} % DONE - NEW - NEW
      data        = generateProblem(401);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-3 * norm(info.b,2);
      info.name   = 'srcsep1_s3';
      accuracy    = 1.0e-10;


   case {47,'seismic_s3'} % Work in progress . . . ORANGE
      data        = generateProblem(901);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-3 * norm(info.b,2);
      info.name   = 'seismic_s3';
      accuracy    = 4.8e-4; % Current: 3.0e-3

   case {48,'phantom2_s3'} % DONE - NEW - NEW
      data        = generateProblem(503);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 1e-3 * norm(info.b,2);
      info.name   = 'phantom2_s3';
      accuracy    = 1e-10;

   case {49,'finger_s3'} % DONE - NEW - NEW
      data        = generateProblem(703);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-3 * norm(info.b,2);
      info.name   = 'finger_s3';
      accuracy    = 1.0e-10;

   case {50,'p3poly_s3'} % DONE - NEW
      data        = generateProblem(6);
      info.A      = data.A;
      info.x0     = data.x0;
      info.b      = data.b;
      info.n      = [];
      info.ortho  = false;
      info.oper   = true;
      info.sigma  = 1e-3 * norm(info.b,2);
      info.name   = 'p3poly_s3';
      accuracy    = 1e-10;


   % ----------------------------------------------------------
   % Partial DCT with compressible x0
   % ----------------------------------------------------------

   case {51,'compress1a'}
      randn('state',0); rand('state',0);
      x0          = randExpDec(4096,10,1e-3); % 1e-3
      info.A      = randDCTMat(1400,4096);
      info.x0     = x0;
      info.b      = info.A * x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.01 * norm(x0,1);
      info.name   = 'compress1a';

   case {52,'compress1b'}
      randn('state',0); rand('state',0);
      x0          = randExpDec(4096,10,1e-4); % 1-e4
      info.A      = randDCTMat(1400,4096);
      info.x0     = x0;
      info.b      = info.A * x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.01 * norm(x0,1); % LARGE
      info.name   = 'compress1b';

   case {53,'compress1c'}
      randn('state',0); rand('state',0);
      x0          = randExpDec(4096,10,1e-4);
      info.A      = randDCTMat(1400,4096);
      info.x0     = x0;
      info.b      = info.A * x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.001 * norm(x0,1); % MODERATELY SMALL
      info.name   = 'compress1c';

   case {54,'compress1d'}
      randn('state',0); rand('state',0);
      x0          = randExpDec(4096,10,1e-4);
      info.A      = randDCTMat(1400,4096);
      info.x0     = x0;
      info.b      = info.A * x0;
      info.n      = [];
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.0001 * norm(x0,1); % VERY SMALL
      info.name   = 'compress1d';
end

% Construct filename for cache
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix, 'cache', filesep];
filename    = [prefixCache, sprintf('problem_bpdn2_%s.mat',lower(info.name))];

% Load problem solution and final parameters
if exist(filename)
   data      = load(filename);   
   lambdas   = data.lambdas;
   sigma     = data.sigma;
   x         = data.x;
   priduaGap = data.priduaGap;
   lambdaGap = data.lambdaGap;
else
   lambdas   = [];
   sigma     = [];
   x         = [];
   priduaGap = 1;
   lambdaGap = 1;
end

if isempty(lambdas)
   % Invoke SPG solver
   options = struct();
   options.verbosity = 3;

   [x,spg_info] = bpdn_spg(info.A,info.b,info.sigma,[],options);

   % Save intermediate result
   lambdas = spg_info.lambdaHist;
   save(filename,'lambdas','sigma','x','priduaGap','lambdaGap');
end

if isempty(sigma) || (priduaGap > accuracy) || (lambdaGap > accuracy)
   % Invoke FISTA solver
   options = struct();
   options.verbosity  = 1;
   options.priduaTol  = accuracy;
   options.lambdaTol  = accuracy;
   options.iterations = Inf;
   options.x0         = x;

   [x,fista_info] = FISTA(info.A,info.b,lambdas(end),options);

   % Save result
   sigma = norm(fista_info.r,2);
   priduaGap = fista_info.priduaGap;
   lambdaGap = fista_info.lambdaGap;
   save(filename,'lambdas','sigma','x','priduaGap','lambdaGap');
end

% Add results to output structure
info.sigma   = sigma;
info.lambdas = lambdas;
info.x       = x;
info.priduaGap = priduaGap;
info.lambdaGap = lambdaGap;
info.accuracy  = accuracy;

% Restore random number generator state
rand('state',state);
randn('state',staten);



% ---------------------------------------------------------------------
function A = randnOrthMat(m,n)
% ---------------------------------------------------------------------
[Q,R] = qr(randn(n,m),0);
A = Q';

% ---------------------------------------------------------------------
function A = randDCTMat(m,n)
% ---------------------------------------------------------------------
p = randperm(n);
A = opRestriction(n,sort(p(1:m))) * opDCT(n);

% ---------------------------------------------------------------------
function x = randnSparseVec(n,k)
% ---------------------------------------------------------------------
x = zeros(n,1);
p = randperm(n);
x(p(1:k)) = randn(k,1);

% ---------------------------------------------------------------------
function x = randExpDec(n,h,l)
% ---------------------------------------------------------------------
p    = randperm(n);
x(p) = exp(linspace(log(h),log(l),n));
x    = x(:);


% ---------------------------------------------------------------------
function [A,x0,n] = problemGaussOne(pct)
% ---------------------------------------------------------------------
% Convex combination of Gaussian and all ones with weight in pct.
randn('state',1);
c = pct / 100;
A = (1-c) * randn(600,1800) + c * ones(600,1800);
x0= zeros(1800,1); x0(1:80) = randn(80,1);
n = 0.03 * randn(600,1);

