function info = problem_mmv(probIdx)
% PROBLEM_MMV  Generates MMV test problems
%
%    INFO = PROBLEM_MMV(IDX) returns problem with index IDX, when IDX
%    is omitted the INFO contains the number of problems.
%
%    INFO has the following fields
%       .A        Measurement matrix
%       .B        Right-hand side
%       .X0       Generating matrix
%       .N        Additive noise vector
%       .oper     Set if A is an implicit operator
%       .sigma    Value for sigma
%       .name     Problem name (use this for saving data, number could change)

% August 21, 2009: Ewout van den Berg

info  = struct();
state = rand('state');
staten= randn('state');

if nargin < 1
   info = 7; % Number of problems
   return;
end

% Set default solution accuracy
accuracy = 1e-10;

switch probIdx
   case {1,'mmv1a'}
      randn('state',1); rand('state',1);

      info.A      = randn(60,100);
      info.X0     = randSupportMat(100,5,6);
      info.B      = info.A * info.X0;
      info.N      = randn(60,5);
      info.N      = info.N * (0.01 * norm(info.B,'fro') / norm(info.N,'fro'));
      info.B      = info.B + info.N;
      info.sigma  = norm(info.N,'fro');
      info.oper   = false;
      info.name   = 'mmv1a';

   case {2,'mmv1b'}
      randn('state',2); rand('state',2);

      info.A      = randn(100,300);
      info.X0     = randSupportMat(300,5,12);
      info.B      = info.A * info.X0;
      info.N      = randn(100,5);
      info.N      = info.N * (0.02 * norm(info.B,'fro') / norm(info.N,'fro'));
      info.B      = info.B + info.N;
      info.sigma  = norm(info.N,'fro') * 0.5;
      info.oper   = false;
      info.name   = 'mmv1b';

   case {3,'mmv1c'}
      randn('state',3); rand('state',3);

      info.A      = randn(60,100);
      info.X0     = randSupportMat(100,10,6);
      info.B      = info.A * info.X0;
      info.N      = randn(60,10);
      info.N      = info.N * (0.02 * norm(info.B,'fro') / norm(info.N,'fro'));
      info.B      = info.B + info.N;
      info.sigma  = norm(info.N,'fro') * 0.3;
      info.oper   = false;
      info.name   = 'mmv1c';

   case {4,'mmv1d'}
      randn('state',4); rand('state',4);

      info.A      = randn(200,400);
      info.X0     = randSupportMat(400,5,20);
      info.B      = info.A * info.X0;
      info.N      = randn(200,5);
      info.N      = info.N * (0.01 * norm(info.B,'fro') / norm(info.N,'fro'));
      info.B      = info.B + info.N;
      info.sigma  = norm(info.N,'fro') * 1.2;
      info.oper   = false;
      info.name   = 'mmv1d';

   case {5,'mmv1e'}
      randn('state',5); rand('state',5);

      info.A      = randn(200,400);
      info.X0     = randSupportMat(400,10,20);
      info.B      = info.A * info.X0;
      info.N      = randn(200,10);
      info.N      = info.N * (0.02 * norm(info.B,'fro') / norm(info.N,'fro'));
      info.B      = info.B + info.N;
      info.sigma  = norm(info.N,'fro');
      info.oper   = false;
      info.name   = 'mmv1e';

   case {6,'mmv1f'}
      randn('state',6); rand('state',6);

      info.A      = randn(300,800);
      info.X0     = randSupportMat(800,3,50);
      info.B      = info.A * info.X0;
      info.N      = randn(300,3);
      info.N      = info.N * (0.01 * norm(info.B,'fro') / norm(info.N,'fro'));
      info.B      = info.B + info.N;
      info.sigma  = norm(info.N,'fro') * 0.9;
      info.oper   = false;
      info.name   = 'mmv1f';

   case {7,'telescope'}
      % Set parameters
      nSensors           = 80;
      nSources           = 8;
      nMeasurements      = 2;
      nRefinements       = 5; % Number of grid refinements
      nDirectionsFine    = 600; % Actual directions
      nDirectionsCoarse  = 100; % Meshed directions
      wavelengthRelative = 5; % Wavelength as a multiple of minimum sensor distance

      % Load sensor array
      [S,distMin] = CirclePacking(nSensors);
      S(3,:) = 0;

      % Compute wavelength based on minimal sensor distance
      wavelength = wavelengthRelative * distMin;

      % Load high-resolution direction-of-arrival grid
      Dfine = PointArrangement(3,2*nDirectionsFine,1e-5);
      idx = Dfine(3,:) >= 0; Dfine = Dfine(:,idx);
      nDfine = size(Dfine,2);

      % Load low-resolution direction-of-arrival grid
      Dcoarse = PointArrangement(3,2*nDirectionsCoarse,1e-5);
      idx = Dcoarse(3,:) >= 0; Dcoarse = Dcoarse(:,idx);
      nDcoarse = size(Dcoarse,2);

      % Add circle at base
      nAngle = ceil(sqrt(nDcoarse));
      Angle = 2*pi*(0:nAngle-1)/nAngle;
      Dcoarse = [Dcoarse,[cos(Angle);sin(Angle);zeros(1,nAngle)]];
      nDcoarse = nDcoarse + nAngle;

      % Compute actual complex shift factors
      Afine   = ComputeShifts3D(S,Dfine,wavelength);

      % Generate sparse complex signal coefficients
      rand('state',nDfine + nSensors); randn('state',nDfine + nSensors);
      p  = randperm(nDfine);
      X0 = zeros(nDfine,nMeasurements);
      X0(p(1:nSources),:) = randn(nSources,nMeasurements) + sqrt(-1) * randn(nSources,nMeasurements);

      % Generate signal
      B = Afine*X0;

      % Compute complex shift factors on grid
      Acoarse = ComputeShifts3D(S,Dcoarse,wavelength);

      info.A      = Acoarse;
      info.X0     = X0;
      info.B      = B;
      info.N      = [];
      info.sigma  = 3;
      info.oper   = false;
      info.name   = 'telescope';

% DO NOT FORGET TO UPDATE PROBLEM COUNT!!!

end




% Construct filename for cache
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix, 'cache', filesep];
filename    = [prefixCache, sprintf('problem_mmv_%s.mat',lower(info.name))];

% Load problem solution and final parameters
if exist(filename)
   data      = load(filename);   
   lambdas   = data.lambdas;
   sigma     = data.sigma;
   X         = data.X;
   priduaGap = data.priduaGap;
   lambdaGap = data.lambdaGap;
else
   lambdas   = [];
   sigma     = [];
   X         = [];
   priduaGap = 1;
   lambdaGap = 1;
end

if isempty(lambdas)
   % Invoke SPG solver
   options = struct();
   options.verbosity = 3;

   [X,spg_info] = mmv_spg(info.A,info.B,info.sigma,[],options);

   % Save intermediate result
   lambdas = spg_info.lambdaHist;
   save(filename,'lambdas','sigma','X','priduaGap','lambdaGap');
end

if isempty(sigma) || (priduaGap > accuracy) || (lambdaGap > accuracy)
   % Invoke FISTA solver
   options = struct();
   options.verbosity  = 1;
   options.priduaTol  = accuracy;
   options.lambdaTol  = accuracy;
   options.iterations = Inf;
   options.X0         = X;

   [x,fista_info] = mmv_fista(info.A,info.B,[],lambdas(end),options);

   % Save result
   sigma = norm(fista_info.R,'fro');
   priduaGap = fista_info.priduaGap;
   lambdaGap = fista_info.lambdaGap;
   save(filename,'lambdas','sigma','X','priduaGap','lambdaGap');
end

% Add results to output structure
info.sigma   = sigma;
info.lambdas = lambdas;
info.X       = X;
info.priduaGap = priduaGap;
info.lambdaGap = lambdaGap;

% Restore random number generator state
rand('state',state);
randn('state',staten);


% ---------------------------------------------------------------------
function X = randSupportMat(m,k,s)
% ---------------------------------------------------------------------
idx = randperm(m);
X = zeros(m,k);
X(idx(1:s),:) = randn(s,k);

