function info = problem_nonneg(probIdx)
% PROBLEM_NONNEG  Generates nonneg. basis pursuit (denoise) test problems
%
%    INFO = PROBLEM_NONNEG(IDX) returns problem with index IDX, when IDX
%    is omitted the INFO contains the number of problems.
%
%    INFO has the following fields
%       .A        Matrix or operator
%       .b        Right-hand side
%       .x0       Generating vector
%       .n        Additive noise vector
%       .ortho    Set if A has orthonormal rows
%       .oper     Set if A is an implicit operator
%       .sigma    Value for sigma
%       .name     Problem name (use this for saving data, number could change)

% October 7, 2009: Ewout van den Berg

info  = struct();
state = rand('state');
staten= randn('state');

if nargin < 1
   info = 7; % Number of problems
   return;
end

% Set default solution accuracy
accuracy = 1e-10;

switch probIdx
   case {1,'nonnegn01'}
      randn('state',1); rand('state',1);

      info.A      = randDCTMat(100,256);
      info.x0     = abs(randnSparseVec(256,10));
      info.b      = info.A * info.x0;
      info.n      = randn(100,1);
      info.n      = info.n * (0.01 * norm(info.b,2) / norm(info.n,2)); % 1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 1.02*norm(info.n,2);
      info.name   = 'nonnegn01';

   case {2,'nonnegn02'}
      randn('state',2); rand('state',2);

      info.A      = randDCTMat(500,8192);
      info.x0     = abs(randnSparseVec(8192,200));
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.001 * norm(info.b,2) / norm(info.n,2)); % .1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.9*norm(info.n,2);
      info.name   = 'nonnegn02';

   case {3,'nonnegn03'} % See also nonnegn02 -- more measurements
      randn('state',2); rand('state',2);

      info.A      = randDCTMat(1500,8192);
      info.x0     = abs(randnSparseVec(8192,200));
      info.b      = info.A * info.x0;
      info.n      = randn(1500,1);
      info.n      = info.n * (0.001 * norm(info.b,2) / norm(info.n,2)); % .1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.9*norm(info.n,2);
      info.name   = 'nonnegn03';

   case {4,'nonnegn04'} % See also nonnegn02 -- more sparse
      randn('state',2); rand('state',2);

      info.A      = randDCTMat(500,8192);
      info.x0     = abs(randnSparseVec(8192,100));
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.001 * norm(info.b,2) / norm(info.n,2)); % .1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.9*norm(info.n,2);
      info.name   = 'nonnegn04';

   case {5,'nonnegn05'} % See also nonnegn02 -- very sparse
      randn('state',2); rand('state',2);

      info.A      = randDCTMat(500,8192);
      info.x0     = abs(randnSparseVec(8192,10));
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.001 * norm(info.b,2) / norm(info.n,2)); % .1 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.9*norm(info.n,2);
      info.name   = 'nonnegn05';


   case {6,'nonnegn06'} % See also nonnegn04 -- noisier
      randn('state',2); rand('state',2);

      info.A      = randDCTMat(500,8192);
      info.x0     = abs(randnSparseVec(8192,100));
      info.b      = info.A * info.x0;
      info.n      = randn(500,1);
      info.n      = info.n * (0.05 * norm(info.b,2) / norm(info.n,2)); % 5 Percent
      info.b      = info.b + info.n;
      info.ortho  = true;
      info.oper   = true;
      info.sigma  = 0.9*norm(info.n,2);
      info.name   = 'nonnegn06';

   case {7,'massspec'}
      data = GenerateSpectMixture;
      info.A     = data.A;
      info.x0    = data.x0;
      info.n     = abs(randn(size(data.A,1),1))*.1;
      info.b     = data.b;
      info.ortho = false;
      info.oper  = false;
      info.sigma = norm(info.n,2);
      info.name  = 'massspec';

% DO NOT FORGET TO UPDATE PROBLEM COUNT!!!

end

if info.sigma > 0
   % Construct filename for cache
   prefix      = mfilepath(mfilename('fullpath'));
   prefixCache = [prefix, 'cache', filesep];
   filename    = [prefixCache, sprintf('problem_nonneg_%s.mat',lower(info.name))];

   % Load problem solution and final parameters
   if exist(filename)
      data      = load(filename);   
      lambdas   = data.lambdas;
      sigma     = data.sigma;
      x         = data.x;
      priduaGap = data.priduaGap;
      lambdaGap = data.lambdaGap;
   else
      lambdas   = [];
      sigma     = [];
      x         = [];
      priduaGap = 1;
      lambdaGap = 1;
   end

   if isempty(lambdas)
      % Invoke SPG solver
      options = struct();
      options.verbosity = 3;

      [x,spg_info] = nonneg_spg(info.A,info.b,info.sigma,[],options);

      % Save intermediate result
      lambdas = spg_info.lambdaHist;
      save(filename,'lambdas','sigma','x','priduaGap','lambdaGap');
   end

   if isempty(sigma) || (priduaGap > accuracy) || (lambdaGap > accuracy)
      % Invoke FISTA solver
      options = struct();
      options.verbosity  = 1;
      options.priduaTol  = accuracy;
      options.lambdaTol  = accuracy;
      options.iterations = Inf;
      options.x0         = x;

      [x,fista_info] = FISTA_NONNEG(info.A,info.b,lambdas(end),options);

      % Save result
      sigma     = norm(fista_info.r,2);
      priduaGap = fista_info.priduaGap;
      lambdaGap = fista_info.lambdaGap;
      save(filename,'lambdas','sigma','x','priduaGap','lambdaGap');
   end

   % Add results to output structure
   info.sigma     = sigma;
   info.lambdas   = lambdas;
   info.x         = x;
   info.priduaGap = priduaGap;
   info.lambdaGap = lambdaGap;
else
   info.x = [];
end








% Restore random number generator state
rand('state',state);
randn('state',staten);

% ---------------------------------------------------------------------
function A = randDCTMat(m,n)
% ---------------------------------------------------------------------
p = randperm(n);
A = opRestriction(n,sort(p(1:m))) * opDCT(n);

% ---------------------------------------------------------------------
function x = randnSparseVec(n,k)
% ---------------------------------------------------------------------
x = zeros(n,1);
p = randperm(n);
x(p(1:k)) = randn(k,1);

