function info = problem_nucnorm(probIdx)
% PROBLEM_NUCNORM  Generates nuclear norm test problems
%
%    INFO = PROBLEM_NUCNORM(IDX) returns problem with index IDX, when IDX
%    is omitted the INFO contains the number of problems.
%
%    INFO has the following fields
%       .M        Full matrix including possible
%       .M0       Full matrix (equal to M if empty)
%       .idx      Observed indices
%       .sigma    Value for sigma
%       .name     Problem name (use this for saving data, number could change)

% August 20, 2009: Ewout van den Berg

info  = struct();
state = rand('state');
staten= randn('state');

if nargin < 1
   info = 12; % Number of problems
   return;
end

% Set default solution accuracy
accuracy = 1e-10;

switch probIdx
   case {1,'nucnorm01'}
      randn('state',1); rand('state',1);

      info.M      = randnRank(10,10,2);
      info.M0     = [];
      info.idx    = randIdx(10,10,80);
      info.sigma  = 0;
      info.name   = 'nucnorm01';
      info.label  = 'nucnorm01';

   case {2,'nucnorm02'}
      randn('state',1); rand('state',1);

      info.M      = randnRank(50,50,4);
      info.M0     = [];
      info.idx    = randIdx(50,50,60);
      info.sigma  = 0;
      info.name   = 'nucnorm02';
      info.label  = 'nucnorm02';

   case {3,'nucnorm03'}
      randn('state',1); rand('state',1);

      info.M      = randnRank(100,100,7);
      info.M0     = [];
      info.idx    = randIdx(100,100,50);
      info.sigma  = 0;
      info.name   = 'nucnorm03';
      info.label  = 'nucnorm03';

   case {4,'nucnorm04'}
      randn('state',1); rand('state',1);

      info.M      = randnRank(100,100,7);
      info.M0     = [];
      info.idx    = randIdx(100,100,30);
      info.sigma  = 0;
      info.name   = 'nucnorm04';
      info.label  = 'nucnorm04';

   case {5,'nucnorm05'}
      randn('state',1); rand('state',1);

      info.M      = randnRank(200,200,12);
      info.M0     = [];
      info.idx    = randIdx(200,200,20);
      info.sigma  = 0;
      info.name   = 'nucnorm05';
      info.label  = 'nucnorm05';

   case {6,'nucnorm06'}
      randn('state',1); rand('state',1);

      info.M      = randnRank(200,200,2);
      info.M0     = [];
      info.idx    = randIdx(200,200,20);
      info.sigma  = 0;
      info.name   = 'nucnorm06';
      info.label  = 'nucnorm06';

   % ----------------------------------------------------------
   % Exactly low-rank but with additive noise
   % ----------------------------------------------------------

   case {7,'nucnormn01'}
      randn('state',1); rand('state',1);

      info.M0     = randnRank(10,10,2);
      info.N      = randn(size(info.M0)) * 0.05;
      info.M      = info.M0 + info.N;
      info.idx    = randIdx(10,10,80);
      info.sigma  = norm(info.N,'fro');
      info.name   = 'nucnormn01';
      info.label  = 'nucnorm01';

   case {8,'nucnormn02'}
      randn('state',1); rand('state',1);

      info.M0     = randnRank(50,50,4);
      info.N      = randn(size(info.M0)) * 0.01;
      info.M      = info.M0 + info.N;
      info.idx    = randIdx(50,50,60);
      info.sigma  = norm(info.N,'fro');
      info.name   = 'nucnormn02';
      info.label  = 'nucnorm02';

   case {9,'nucnormn03'}
      randn('state',1); rand('state',1);

      info.M0     = randnRank(100,100,7);
      info.N      = randn(size(info.M0)) * 0.02;
      info.M      = info.M0 + info.N;
      info.idx    = randIdx(100,100,50);
      info.sigma  = norm(info.N,'fro');
      info.name   = 'nucnormn03';
      info.label  = 'nucnorm03';

   case {10,'nucnormn04'}
      randn('state',1); rand('state',1);

      info.M0     = randnRank(100,100,7);
      info.N      = randn(size(info.M0)) * 0.03;
      info.M      = info.M0 + info.N;
      info.idx    = randIdx(100,100,30);
      info.sigma  = norm(info.N,'fro');
      info.name   = 'nucnormn04';
      info.label  = 'nucnorm04';

   case {11,'nucnormn05'}
      randn('state',1); rand('state',1);

      info.M0     = randnRank(200,200,12);
      info.N      = randn(size(info.M0)) * 0.01;
      info.M      = info.M0 + info.N;
      info.idx    = randIdx(200,200,20);
      info.sigma  = norm(info.N,'fro');
      info.name   = 'nucnormn05';
      info.label  = 'nucnorm05';

   case {12,'nucnormn06'}
      randn('state',1); rand('state',1);

      info.M0     = randnRank(200,200,2);
      info.N      = randn(size(info.M0)) * 0.01;
      info.M      = info.M0 + info.N;
      info.idx    = randIdx(200,200,20);
      info.sigma  = norm(info.N,'fro');
      info.name   = 'nucnormn06';
      info.label  = 'nucnorm06';

end

if info.sigma > 0
   % Construct filename for cache
   prefix      = mfilepath(mfilename('fullpath'));
   prefixCache = [prefix, 'cache', filesep];
   filename    = [prefixCache, sprintf('problem_nucnorm_%s.mat',lower(info.name))];

   % Load problem solution and final parameters
   if exist(filename)
      data      = load(filename);   
      lambdas   = data.lambdas;
      sigma     = data.sigma;
      X         = data.X;
      priduaGap = data.priduaGap;
      lambdaGap = data.lambdaGap;
   else
      lambdas   = [];
      sigma     = [];
      X         = [];
      priduaGap = 1;
      lambdaGap = 1;
   end

   if isempty(lambdas)
      % Invoke SPG solver
      options = struct();
      options.verbosity = 3;

      [X,spg_info] = nucnorm_spg(info.M,info.idx,info.sigma,[],options);

      % Save intermediate result
      lambdas = spg_info.lambdaHist;
      save(filename,'lambdas','sigma','X','priduaGap','lambdaGap');
   end

   if isempty(sigma) || (priduaGap > accuracy) || (lambdaGap > accuracy)
      % Invoke FISTA solver
      options = struct();
      options.verbosity  = 1;
      options.priduaTol  = accuracy;
      options.lambdaTol  = accuracy;
      options.iterations = Inf;
      options.X0         = X;

      [X,fista_info] = FISTA_NUCNORM(info.M,info.idx,lambdas(end),options);

      % Save result
      sigma     = norm(fista_info.R,'fro');
      priduaGap = fista_info.priduaGap;
      lambdaGap = fista_info.lambdaGap;
      save(filename,'lambdas','sigma','X','priduaGap','lambdaGap');
   end

   % Add results to output structure
   info.sigma     = sigma;
   info.lambdas   = lambdas;
   info.X         = X;
   info.priduaGap = priduaGap;
   info.lambdaGap = lambdaGap;
else
   info.lambdas = [NaN];
   info.X = [];
end








% Restore random number generator state
rand('state',state);
randn('state',staten);



% ---------------------------------------------------------------------
function M = randnRank(m,n,r)
% ---------------------------------------------------------------------
M = randn(m,r) * randn(r,n);

% ---------------------------------------------------------------------
function idx = randIdx(m,n,percentage)
% ---------------------------------------------------------------------
idx = randperm(m*n);
k   = min(m*n,max(1,round(m*n*percentage/100)));
idx = sort(idx(1:k));

% ---------------------------------------------------------------------
function nn = nuclearNorm(A)
% ---------------------------------------------------------------------
[U,S,V] = svd(A,'econ');
nn = norm(diag(S),1);

