function table_bp(latexFlag,outputFlag)

% Check arguments
if nargin < 1, latexFlag = false; end;
if nargin < 2, outputFlag = false; end;

% Get path information
prefix      = mfilepath(mfilename('fullpath'));
prefixTable = [prefix,'..',filesep,'..',filesep,'fig',filesep];

problems  = {'sgnspike','zsgnspike','dcthdr','angiogram','jitter','blkheavi'};
abbrev    = {'sgnspike','zsgnspike','dcthdr','angiogr.'  ,'jitter','blkheavi'};
solvers   = solvers_bp();
solverIdx = [1,2,3,5,4,6,7];

if latexFlag
   solverName = {'\cvxsdp','\nesta','\spg','\spgt','\pqn','Homotopy','Bregman'};
else
   solverName = {'CVX/SDPT3','Nesta','SPOR-SPG-H','SPOR-SPG','SPOR-PQN','Homotopy','Bregman'};
end

if outputFlag
   filename = [prefixTable,'TableBP.tex'];
   fp = fopen(filename,'w');
   if fp == -1, error('Could not open output file %s\n', filename); end;
else
   fp = 1;
end

% Output header
if latexFlag
   fprintf(fp,'\\begin{tabular}{lllrlll}\n');
   fprintf(fp,'\\toprule\n');
   fprintf(fp,'\\multicolumn{1}{l}{} & Solver & Time (s) & Aprod & $\\norm{x}_1$ & $\\norm{r}_2$ & $\\norm{x - x_0}_\\infty$ \\\\\n');
else
   formatStr = '%-10s  %-8s  %8s  %-9s  %-9s  %-9s\n';
   headerFld = {'Solver','Time(s)','Aprod','norm(x,1)','norm(r,2)','||x-x0||'};
   headerStr = sprintf(formatStr, headerFld{:});
   fprintf(fp,'%s\n',repmat('=',1,length(headerStr)));
   fprintf(fp,'%s',headerStr);
   fprintf(fp,'%s\n',repmat('=',1,length(headerStr)));
end

% Run all problems
for i=1:length(problems)
   problemAbbr = abbrev{i};
   problemName = problems{i};
   problem = problem_bp(problemName);

   % Run all solvers first
   solverInfo = {};
   for j=1:length(solverIdx)
      [x,info,status] = solution_bp(problem,solvers(solverIdx(j),:));

      if ~strcmp(status,'ok'), continue; end;

      strTime = sprintf('%6.1e', info.time);
      if isnan(info.nA+info.nAT)
         strAprod = 'n.a.';
      else
         strAprod = sprintf('%d',info.nA+info.nAT);
      end
      strXNorm = sprintf('%9.4e', info.xNorm);
      strRNorm = sprintf('%6.1e', info.rNorm);
      strDNorm = sprintf('%6.1e', norm(x-problem.x0,inf));

      data = {solverName{j}, strTime, strAprod, strXNorm, strRNorm, strDNorm};
      solverInfo{end+1} = data;
   end

   % Format output
   [solverInfo,widths] = formatTableData(solverInfo,latexFlag);

   % Output problem header
   if latexFlag
      fprintf(fp,'\\midrule\n');
      fprintf(fp,'\\multirow{%d}{*}{\\rotatebox{90}{\\small %s}}\n', length(solverInfo), problemAbbr);
   else
      fprintf(fp,'%s[%s]%s\n',repmat('-',1,3),problemName,repmat('-',1,max(0,length(headerStr)-5-length(problemName))));
   end

   % Output problem data
   for j=1:length(solverInfo)
      data = solverInfo{j};

      if latexFlag
         fprintf(fp,'&%*s & %*s & %*s & %*s & %*s & %*s \\\\\n', ...
                 -widths(1), data{1}, -widths(2), data{2}, widths(3), data{3}, ...
                 -widths(4), data{4}, -widths(5), data{5},-widths(6), data{6});
      else
         fprintf(fp,formatStr,data{:});
      end
   end

end

% Output table footer
if latexFlag
   fprintf(fp,'\\bottomrule\n');
   fprintf(fp,'\\end{tabular}\n');
end

% Close file
if outputFlag
   fclose(fp);
end

