function table_nucnorm(problems,filename,latexFlag,outputFlag)

% Check arguments
if nargin < 3, latexFlag = false; end;
if nargin < 4, outputFlag = false; end;

% Get path information
prefix      = mfilepath(mfilename('fullpath'));
prefixTable = [prefix,'..',filesep,'..',filesep,'fig',filesep];

solvers   = solvers_nucnorm();
solverIdx = [1,5,7,6,2,3,8];

if latexFlag
   solverName = {'\cvxsdp','\spg','\spgt','\pqn','\fpca','\svt','\fista'};
else
   solverName = {'CVX/SDPT3','SPOR-SPG-H','SPOR-SPG','SPOR-PQN','FPCA','SVT','FISTA'};
end

if outputFlag
   filename = [prefixTable,filename];
   fp = fopen(filename,'w');
   if fp == -1, error('Could not open output file %s\n', filename); end;
else
   fp = 1;
end

% Output header
if latexFlag
   fprintf(fp,'\\begin{tabular}{lllrllc}\n');
   fprintf(fp,'\\toprule\n');
   fprintf(fp,'\\multicolumn{1}{l}{} & Solver & Time (s) & \\#SVD & $\\norm{X}_n$ & $\\frac{\\abs{\\norm{r}_2-\\sigma}}{\\max(1,\\sigma)}$ & $\\norm{X_: - X^*_:}_\\infty$\\\\\n');
   fprintf(fp,'\\midrule\n');
else
   formatStr = '%-10s  %-8s  %8s  %-9s  %-9s  %-9s  %8s\n';
   headerFld = {'Solver','Time(s)','#SVD','norm(x,*)','||r||_sig','||x-x0||','rank'};
   headerStr = sprintf(formatStr, headerFld{:});
   fprintf(fp,'%s\n',repmat('=',1,length(headerStr)));
   fprintf(fp,'%s\n',headerStr);
   fprintf(fp,'%s\n',repmat('=',1,length(headerStr)));
end

for i=1:length(problems)
   problemInfo = {};
   problemName = problems{i};

   if isnan(problemName)
      if latexFlag
         fprintf(fp,'\\midrule\n');
      end
      continue;
   else
      problem = problem_nucnorm(problemName);
      problemLabel = problem.label;
      XRef = problem.X;
   end

   for j=1:length(solverIdx)
      % Get results
      [X,info,status] = solution_nucnorm(problem,solvers(solverIdx(j),:));
      if ~strcmp(status,'ok'), continue; end;

      if (j==1) && isempty(XRef)
         XRef = X;
         XRefFlag = true;
      else
         XRefFlag = false;
      end

      % Format and append results
      strTime  = sprintf('%6.1e',info.time);
      if ~isfield(info,'nProj')
         strAprod = 'n.a.';
      else
         strAprod = sprintf('%d',info.nProj);
      end
      strXNorm = sprintf('%9.4e',norm_nuc(X));
      strRNorm = sprintf('%6.1e',abs(norm(info.r)-problem.sigma) / max(1,problem.sigma));
      if ~XRefFlag
%         strDNorm = sprintf('%6.1e',norm(X-XRef,inf));
          strDNorm = sprintf('%6.1e',norm(X(:)-XRef(:),inf));
      else
         strDNorm = '---';
      end
      
      s = svd(X);
%     strRank = sprintf('%6d', sum(1.0 * (s > 1e-4)));
      strRank = sprintf('%6d', rank(X));
%     strRank = sprintf('%6d', sparsity(s));
      
      data = {solverName{j},strTime,strAprod,strXNorm,strRNorm,strDNorm,strRank};
      problemInfo{end+1} = data;
   end

   % Format output data
   [problemInfo,widths] = formatTableData(problemInfo,latexFlag);

   % Count number of separator fields
   nSeps = 0;
   for j=1:length(problemInfo)
      data = problemInfo{j};
      if isempty(data{1}), nSeps = nSeps + 1; end;
   end

   % Output problem header
   if latexFlag
      fprintf(fp,'\\multirow{%d}{*}{\\rotatebox{90}{\\small %s}}\n', length(problemInfo)-nSeps, problemLabel);
   else
      fprintf(fp,'%s[%s]%s\n',repmat('-',1,3),problemLabel,repmat('-',1,max(0,length(headerStr)-5-length(problemLabel))));
   end

   % Output problem data
   for j=1:length(problemInfo)
      data = problemInfo{j};
      if isempty(data{1})
         if latexFlag
            fprintf(fp,'\\cline{3-7}\n');
         end
      else
         if latexFlag
            fprintf(fp,'&%*s & %*s & %*s & %*s & %*s & %*s\\\\\n', ...
                    -widths(1), data{1}, -widths(2), data{2}, widths(3), data{3}, ...
                    -widths(4), data{4}, -widths(5), data{5},-widths(6), data{6}  ...
                    );%,-widths(7), data{7});
         else
            fprintf(fp,formatStr,data{:});
         end
      end
   end

   % Output problem footer
   if latexFlag
      if (i ~= length(problems))
         fprintf(fp,'\\midrule\n');
      else
         fprintf(fp,'\\bottomrule\n');
      end
   end
end


% Output table footer
if latexFlag
   fprintf(fp,'\\end{tabular}\n');
end

% Close file
if outputFlag
   fclose(fp);
end

